"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryComplexityPlugin = void 0;
const graphql_1 = require("graphql");
const graphql_query_complexity_1 = require("graphql-query-complexity");
const errors_1 = require("../errors");
const DEFAULT_FIELD_COMPLEXITY = 1;
exports.queryComplexityPlugin = (schema, logger, maximumComplexity) => ({
    requestDidStart: () => ({
        didResolveOperation({ request, document }) {
            const complexity = graphql_query_complexity_1.getComplexity({
                schema,
                query: request.operationName
                    ? graphql_1.separateOperations(document)[request.operationName]
                    : document,
                variables: request.variables,
                estimators: [
                    graphql_query_complexity_1.directiveEstimator({ name: 'complexity' }),
                    graphql_query_complexity_1.fieldExtensionsEstimator(),
                    graphql_query_complexity_1.simpleEstimator({ defaultComplexity: DEFAULT_FIELD_COMPLEXITY })
                ]
            });
            if (complexity > maximumComplexity) {
                throw new errors_1.QueryTooComplex(complexity, maximumComplexity);
            }
            logger.trace('Used query complexity points:', complexity);
        }
    })
});
//# sourceMappingURL=query_complexity_plugin.js.map