"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getConfig = void 0;
const errors_1 = require("./errors");
const fs_extra_1 = __importDefault(require("fs-extra"));
function getConfig() {
    return __awaiter(this, void 0, void 0, function* () {
        const env = filterAndTypecastEnvs(process.env);
        if (!env.cardanoNodeConfigPath) {
            throw new errors_1.MissingConfig('CARDANO_NODE_CONFIG_PATH env not set');
        }
        if (!env.hasuraCliPath) {
            throw new errors_1.MissingConfig('HASURA_CLI_PATH env not set');
        }
        if (!env.hasuraUri) {
            throw new errors_1.MissingConfig('HASURA_URI env not set');
        }
        if (!env.postgres.dbFile && !env.postgres.db) {
            throw new errors_1.MissingConfig('POSTGRES_DB_FILE or POSTGRES_DB env not set');
        }
        if (!env.postgres.host) {
            throw new errors_1.MissingConfig('POSTGRES_HOST env not set');
        }
        if (!env.postgres.passwordFile && !env.postgres.password) {
            throw new errors_1.MissingConfig('POSTGRES_PASSWORD_FILE or POSTGRES_PASSWORD env not set');
        }
        if (!env.postgres.port) {
            throw new errors_1.MissingConfig('POSTGRES_PORT env not set');
        }
        if (!env.postgres.userFile && !env.postgres.user) {
            throw new errors_1.MissingConfig('POSTGRES_USER_FILE or POSTGRES_USER env not set');
        }
        let db;
        try {
            db = {
                database: env.postgres.db || (yield fs_extra_1.default.readFile(env.postgres.dbFile, 'utf8')).toString(),
                host: env.postgres.host,
                password: env.postgres.password || (yield fs_extra_1.default.readFile(env.postgres.passwordFile, 'utf8')).toString(),
                port: env.postgres.port,
                user: env.postgres.user || (yield fs_extra_1.default.readFile(env.postgres.userFile, 'utf8')).toString()
            };
        }
        catch (error) {
            throw new errors_1.MissingConfig('Database configuration cannot be read');
        }
        const { postgres } = env, selectedEnv = __rest(env, ["postgres"]);
        return Object.assign(Object.assign({}, selectedEnv), { allowIntrospection: (process.env.NODE_ENV === 'production' && env.allowIntrospection) ||
                (process.env.NODE_ENV !== 'production' && env.allowIntrospection !== false), allowedOrigins: env.allowedOrigins || true, apiPort: env.apiPort || 3100, cacheEnabled: env.cacheEnabled || false, db, loggerMinSeverity: env.loggerMinSeverity || 'info', listenAddress: env.listenAddress || '0.0.0.0', maxQueryComplexity: env.maxQueryComplexity, pollingInterval: {
                adaSupply: env.pollingInterval.adaSupply || 1000 * 60
            }, queryDepthLimit: env.queryDepthLimit || 10 });
    });
}
exports.getConfig = getConfig;
function filterAndTypecastEnvs(env) {
    const { ALLOW_INTROSPECTION, ALLOWED_ORIGINS, ALLOW_LIST_PATH, API_PORT, ASSET_METADATA_UPDATE_INTERVAL, CACHE_ENABLED, CARDANO_NODE_CONFIG_PATH, HASURA_CLI_PATH, HASURA_URI, LISTEN_ADDRESS, LOGGER_MIN_SEVERITY, MAX_QUERY_COMPLEXITY, METADATA_SERVER_URI, OGMIOS_HOST, OGMIOS_PORT, POLLING_INTERVAL_ADA_SUPPLY, POSTGRES_DB, POSTGRES_DB_FILE, POSTGRES_HOST, POSTGRES_PASSWORD, POSTGRES_PASSWORD_FILE, POSTGRES_PORT, POSTGRES_USER, POSTGRES_USER_FILE, PROMETHEUS_METRICS, QUERY_DEPTH_LIMIT, TRACING } = env;
    return {
        allowIntrospection: ALLOW_INTROSPECTION === 'true',
        allowedOrigins: ALLOWED_ORIGINS,
        allowListPath: ALLOW_LIST_PATH,
        apiPort: Number(API_PORT),
        cacheEnabled: CACHE_ENABLED === 'true',
        cardanoNodeConfigPath: CARDANO_NODE_CONFIG_PATH,
        hasuraCliPath: HASURA_CLI_PATH,
        hasuraUri: HASURA_URI,
        listenAddress: LISTEN_ADDRESS,
        loggerMinSeverity: LOGGER_MIN_SEVERITY,
        maxQueryComplexity: Number(MAX_QUERY_COMPLEXITY),
        metadataUpdateInterval: {
            assets: ASSET_METADATA_UPDATE_INTERVAL ? Number(ASSET_METADATA_UPDATE_INTERVAL) : undefined
        },
        metadataServerUri: METADATA_SERVER_URI,
        ogmios: {
            host: OGMIOS_HOST,
            port: OGMIOS_PORT ? Number(OGMIOS_PORT) : undefined
        },
        pollingInterval: {
            adaSupply: Number(POLLING_INTERVAL_ADA_SUPPLY)
        },
        postgres: {
            db: POSTGRES_DB,
            dbFile: POSTGRES_DB_FILE,
            host: POSTGRES_HOST,
            password: POSTGRES_PASSWORD,
            passwordFile: POSTGRES_PASSWORD_FILE,
            port: POSTGRES_PORT ? Number(POSTGRES_PORT) : undefined,
            user: POSTGRES_USER,
            userFile: POSTGRES_USER_FILE
        },
        prometheusMetrics: PROMETHEUS_METRICS === 'true',
        queryDepthLimit: Number(QUERY_DEPTH_LIMIT),
        tracing: TRACING === 'true'
    };
}
//# sourceMappingURL=config.js.map