"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !exports.hasOwnProperty(p)) __createBinding(exports, m, p);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bunyan_1 = require("bunyan");
const config_1 = require("./config");
const api_cardano_db_hasura_1 = require("@cardano-graphql/api-cardano-db-hasura");
const util_1 = require("@cardano-graphql/util");
const death_1 = __importDefault(require("death"));
const path_1 = __importDefault(require("path"));
const Server_1 = require("./Server");
__exportStar(require("./config"), exports);
(function () {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const config = yield config_1.getConfig();
        const logger = bunyan_1.createLogger({
            name: 'cardano-graphql',
            level: config.loggerMinSeverity
        });
        try {
            const loadGenesis = (eraName) => require(path_1.default.resolve(path_1.default.dirname(config.cardanoNodeConfigPath), cardanoNodeConfig[`${eraName}GenesisFile`]));
            const schemas = [];
            const cardanoNodeConfig = require(config.cardanoNodeConfigPath);
            const genesis = {
                alonzo: loadGenesis('Alonzo'),
                byron: loadGenesis('Byron'),
                shelley: loadGenesis('Shelley')
            };
            const lastConfiguredMajorVersion = require(config.cardanoNodeConfigPath)['LastKnownBlockVersion-Major'];
            const cardanoNodeClient = new api_cardano_db_hasura_1.CardanoNodeClient(lastConfiguredMajorVersion, logger);
            const hasuraClient = new api_cardano_db_hasura_1.HasuraClient(config.hasuraCliPath, config.hasuraUri, config.pollingInterval.adaSupply, lastConfiguredMajorVersion, logger);
            const db = new api_cardano_db_hasura_1.Db(config.db, logger);
            const chainFollower = new api_cardano_db_hasura_1.ChainFollower(hasuraClient, logger, config.db);
            const metadataClient = new api_cardano_db_hasura_1.MetadataClient(config.metadataServerUri);
            const worker = new api_cardano_db_hasura_1.Worker(hasuraClient, logger, metadataClient, config.db, {
                metadataUpdateInterval: {
                    assets: (_a = config.metadataUpdateInterval) === null || _a === void 0 ? void 0 : _a.assets
                }
            });
            const server = new Server_1.Server(schemas, config, logger);
            schemas.push(yield api_cardano_db_hasura_1.buildSchema(hasuraClient, genesis, cardanoNodeClient));
            yield db.init({
                onDbInit: () => __awaiter(this, void 0, void 0, function* () {
                    yield Promise.all([
                        hasuraClient.shutdown,
                        cardanoNodeClient.shutdown,
                        worker.shutdown,
                        chainFollower.shutdown
                    ]);
                    yield server.shutdown();
                }),
                onDbSetup: () => __awaiter(this, void 0, void 0, function* () {
                    try {
                        yield server.init();
                        yield hasuraClient.initialize();
                        yield cardanoNodeClient.initialize(config.ogmios);
                        yield metadataClient.initialize();
                        yield chainFollower.initialize(config.ogmios);
                        const mostRecentPoint = yield hasuraClient.getMostRecentPointWithNewAsset();
                        const points = mostRecentPoint !== null ? [mostRecentPoint, 'origin'] : ['origin'];
                        yield worker.start();
                        yield chainFollower.start(points);
                        yield server.start();
                    }
                    catch (error) {
                        logger.error(error.message);
                        if (error instanceof util_1.errors.HostDoesNotExist) {
                            process.exit(1);
                        }
                    }
                })
            });
            death_1.default(() => __awaiter(this, void 0, void 0, function* () {
                yield Promise.all([
                    hasuraClient.shutdown,
                    cardanoNodeClient.shutdown,
                    worker.shutdown,
                    chainFollower.shutdown
                ]);
                yield db.shutdown();
                yield server.shutdown();
                process.exit(1);
            }));
        }
        catch (error) {
            logger.error(error);
            process.exit(1);
        }
    });
})();
//# sourceMappingURL=index.js.map